/**
 * Copyright (c) 2006 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * History:
 *
 * $Log: AtgGeschwindigkeitsKlassenLangzeitdaten.java,v $
 * Revision 1.2  2007/07/25 13:02:10  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:42:19  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.3  2006/11/13 09:24:12  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2006/08/29 08:54:19  Schawaller
 * neue methode: alle ATL als Liste
 *
 * Revision 1.1  2006/08/28 13:43:39  Schawaller
 * Neue klasse zum kapseln von ATG
 *
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 *
 * Die abstrakte Klasse stellt Methoden für die Klassen bereit, die
 * Attribut-Gruppen vom Typ GeschwindigkeitsKlassen kapseln.
 *
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 *
 * <p>
 * Organisation: Logos GmbH Rostock (www.logos-ib.de)
 * </p>
 *
 * @author J.Schawaller
 * @version 1.0
 */
public abstract class AtgGeschwindigkeitsKlassenLangzeitdaten {

	// PID der Atg wird durch die implementierung der abstrakten methode
	// ermittelt
	private final String ATG_PID_LangZeitDatenGeschwindigkeitsKlassen = getAtgPid();

	/**
	 * _Data - Daten der ATG
	 */
	private final Data _Data;

	/**
	 * __TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen - Attributgruppe der Daten
	 */
	private AttributeGroup __TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen;

	/**
	 * ATT_TYP_LangzeitdatenGeschwindigkeit - Typ der Attribute
	 */
	private static final String ATT_TYP_LANGZEITDATEN_FAHRZEUGANZAHL = "LangzeitdatenFahrzeuganzahl";

	/**
	 * ATTR_NAME_KFZ - Attribute Name f&uuml;r Fahrzeugart Kfz
	 */
	public static final String ATTR_NAME_KFZ = "Kfz";
	/**
	 * ATTR_NAME_PKWAE - Attribute Name f&uuml;r Fahrzeugart Pkw&Auml;
	 */
	public static final String ATTR_NAME_PKWAE = "PkwÄ";
	/**
	 * ATTR_NAME_LKWAE - Attribute Name f&uuml;r Fahrzeugart Lkw&Auml;
	 */
	public static final String ATTR_NAME_LKWAE = "LkwÄ";
	/**
	 * ATTR_NAME_KFZNK - Attribute-Name f&uuml;r Fahrzeugart KfzNk
	 */
	public static final String ATTR_NAME_KFZNK = "KfzNk";
	/**
	 * ATTR_NAME_PKWG - Attribute-Name f&uuml;r Fahrzeugart PkwG
	 */
	public static final String ATTR_NAME_PKWG = "PkwG";
	/**
	 * ATTR_NAME_PKW - Attribute-Name f&uuml;r Fahrzeugart Pkw
	 */
	public static final String ATTR_NAME_PKW = "Pkw";
	/**
	 * ATTR_NAME_KRAD - Attribute-Name f&uuml;r Fahrzeugart Krad
	 */
	public static final String ATTR_NAME_KRAD = "Krad";
	/**
	 * ATTR_NAME_LFW - Attribute-Name f&uuml;r Fahrzeugart Lfw
	 */
	public static final String ATTR_NAME_LFW = "Lfw";
	/**
	 * ATTR_NAME_PKWA - Attribute-Name f&uuml;r Fahrzeugart PkwA
	 */
	public static final String ATTR_NAME_PKWA = "PkwA";
	/**
	 * ATTR_NAME_LKW - Attribute-Name f&uuml;r Fahrzeugart Lkw
	 */
	public static final String ATTR_NAME_LKW = "Lkw";
	/**
	 * ATTR_NAME_BUS - Attribute-Name f&uuml;r Fahrzeugart Bus
	 */
	public static final String ATTR_NAME_BUS = "Bus";
	/**
	 * ATTR_NAME_LKWK - Attribute-Name f&uuml;r Fahrzeugart LkwK
	 */
	public static final String ATTR_NAME_LKWK = "LkwK";
	/**
	 * ATTR_NAME_LKWA - Attribute-Name f&uuml;r Fahrzeugart LkwA
	 */
	public static final String ATTR_NAME_LKWA = "LkwA";
	/**
	 * ATTR_NAME_SATTELKFZ - Attribute-Name f&uuml;r Fahrzeugart SattelKfz
	 */
	public static final String ATTR_NAME_SATTELKFZ = "SattelKfz";

	/**
	 * __ATTR_NAMEN - Namen der Attribute als Array zusammengefa&szlig;t.
	 */
	private static final String[] ATTR_NAMEN = {
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KFZ,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWAE,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWAE,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KFZNK,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWG,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKW,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KRAD,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LFW,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWA,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKW,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_BUS,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWK,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWA,
		AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_SATTELKFZ };

	/**
	 * Erzeugt eine neue Instanz mit dem übergebenem Data.
	 *
	 * @param data
	 *            Data
	 */
	public AtgGeschwindigkeitsKlassenLangzeitdaten(final Data data) {
		this._Data = data;
	}

	/**
	 * Erzeugt eine neue Instanz und erstellt ein Data.
	 *
	 */
	public AtgGeschwindigkeitsKlassenLangzeitdaten() {
		if (__TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen == null) {
			__TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen = LogosClientDavConnection
					.getInstance()
					.getDataModel()
					.getAttributeGroup(
							ATG_PID_LangZeitDatenGeschwindigkeitsKlassen);
		}
		_Data = LogosClientDavConnection.getInstance().createData(
				__TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen);
	}

	/**
	 * Gibt die PID der speziellen ATG wieder.
	 *
	 * @return String PID der Atg
	 */
	public abstract String getAtgPid();

	/**
	 * Initialisiert die Werte des Data für den generichen Zugriff.
	 *
	 */
	public void doInitialize() {
		if (__TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen == null) {
			__TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen = LogosClientDavConnection
					.getInstance()
					.getDataModel()
					.getAttributeGroup(
							ATG_PID_LangZeitDatenGeschwindigkeitsKlassen);
		}
		final Iterator<Attribute> attrIterator = __TYP_ATG_LangzeitdatenGeschwindigkeitsKlassen
				.getAttributes().iterator();
		while (attrIterator.hasNext()) {
			final Attribute oAttribute = attrIterator.next();
			if (AtgGeschwindigkeitsKlassenLangzeitdaten.ATT_TYP_LANGZEITDATEN_FAHRZEUGANZAHL
					.equals(oAttribute.getAttributeType().getName())) {
				final Data.Array oArray = _Data.getItem(oAttribute.getName())
						.asArray();
				for (int i = 0; i < oArray.getLength(); i++) {
					new AtlLangZeitDatenFahrzeugAnzahl(oArray.getItem(i))
					.doInitialize();
				}
			}
		}
	}

	/**
	 * Gibt das Data wieder.
	 *
	 * @return Data
	 */
	public Data getData() {
		return _Data;
	}

	/*
	 * Getter - Methoden
	 */

	/**
	 * Gibt die Daten aller AttributListen-Arrays als Liste wieder. Ein
	 * Listeneintrag enth&auml;lt ein Array von
	 * {@link de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtlLangZeitDatenFahrzeugAnzahl}
	 * f&uuml;r die einzelenen Geschwindigkeitsklassen einer Fahrzeugart.
	 *
	 * @return List Daten aller AttributListen-Arrays
	 */
	public List<AtlLangZeitDatenFahrzeugAnzahl[]> getAttributListenLangZeitDatenFahrzeuganzahl() {
		final List<AtlLangZeitDatenFahrzeugAnzahl[]> result = new ArrayList<>(
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAMEN.length);
		for (final String attrName : AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAMEN) {
			result.add(getAnzahlKlassen(attrName));
		}
		return result;
	}

	/**
	 * Die Methode gibt abhängig vom Parameter das Array der entsprechenden
	 * AttributListe wieder.
	 *
	 * @param fahrzeugArt
	 *            String Attribut-Name der Liste
	 * @return AtlLangZeitDatenFahrzeugAnzahl[] Array von AttributListen
	 */
	private AtlLangZeitDatenFahrzeugAnzahl[] getAnzahlKlassen(
			final String fahrzeugArt) {
		AtlLangZeitDatenFahrzeugAnzahl[] retValue;
		final Data.Array oArray = _Data.getItem(fahrzeugArt).asArray();
		if (oArray.getLength() > 0) {
			retValue = new AtlLangZeitDatenFahrzeugAnzahl[oArray.getLength()];
			for (int i = 0; i < oArray.getLength(); i++) {
				retValue[i] = new AtlLangZeitDatenFahrzeugAnzahl(
						oArray.getItem(i));
			}
		} else {
			retValue = null;
		}
		return retValue;
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getKfzGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KFZ);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getPkwAeGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWAE);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getLkwAeGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWAE);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getKfzNkGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KFZNK);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getPkwGGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWG);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getPkwGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKW);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getKradGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KRAD);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getLfwGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LFW);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getPkwAGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWA);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getLkwGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKW);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getBusGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_BUS);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getLkwKGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWK);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getLkwAGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWA);
	}

	public AtlLangZeitDatenFahrzeugAnzahl[] getSattelKfzGeschwKlassen() {
		return getAnzahlKlassen(AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_SATTELKFZ);
	}

	/**
	 * Gibt die Attribut-Liste f&uuml;r die entsprechenden Parameter wieder.
	 *
	 * @param itemNo
	 *            int Nummer der Geschwindigkeitsklasse
	 * @param fahrzeugArt
	 *            String Name der AttributListe (Fahrzeugart)
	 * @return AtlLangZeitDatenFahrzeugAnzahl Attribut-Liste mit Anzahl in der
	 *         V-Klasse
	 */
	private AtlLangZeitDatenFahrzeugAnzahl getAnzahlKlasse(final int itemNo,
			final String fahrzeugArt) {
		AtlLangZeitDatenFahrzeugAnzahl retValue;
		final Data.Array oArray = _Data.getItem(fahrzeugArt).asArray();
		if (oArray.getLength() > itemNo) {
			retValue = new AtlLangZeitDatenFahrzeugAnzahl(
					oArray.getItem(itemNo));
		} else {
			retValue = null;
		}
		return retValue;
	}

	public AtlLangZeitDatenFahrzeugAnzahl getKfzGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KFZ);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkwAEGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWAE);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkwAEGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWAE);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getKfzNkGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KFZNK);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkwGGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWG);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkwGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKW);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getKradGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_KRAD);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLfwGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LFW);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkwAGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_PKWA);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkwGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKW);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getBusGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_BUS);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkwKGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWK);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkwAGeschwKlasse(final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_LKWA);
	}

	public AtlLangZeitDatenFahrzeugAnzahl getSattelKfzGeschwKlasse(
			final int itemNo) {
		return getAnzahlKlasse(itemNo,
				AtgGeschwindigkeitsKlassenLangzeitdaten.ATTR_NAME_SATTELKFZ);
	}

	/**
	 * Gibt die Eigenschaften der Klasse als String wieder.
	 *
	 * @return String Eigenschaften der Klasse
	 */
	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("atg[").append(ATG_PID_LangZeitDatenGeschwindigkeitsKlassen)
		.append("]: ");
		sb.append(" data[").append(_Data).append("]");
		return sb.toString();
	}

}
