/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * Projektbeschreibung:
 * @title
 * @version 1.0
 * @copyright Copyright (c) 2004
 */

/**
 * kapselt den DatenModellTyp verkehrsDatenQuelleASC
 */
public class FahrstreifenQuelleASCII extends ConfigurationObjectWrapper implements FahrstreifenQuelle {

	private static ConfigurationObjectType __ConfigurationObjectType;
	private static AttributeGroup __AtgVerkehrsDatenQuelleASC;

	private String _PID;
	private String _URL;
	private String _DatenFormat;
	private String _VKRaster;
	private int _FahrStreifenIndex;

	public String getURL() {
		return _URL;
	}

	public int getFahrStreifenIndex() {
		return _FahrStreifenIndex;
	}

	public String getDatenFormat() {
		return _DatenFormat;
	}

	public String getVKRaster() {
		return _VKRaster;
	}

	protected FahrstreifenQuelleASCII(final String pid)
			throws DataModellMismatchException, DataModellObjectNotFoundException {
		super(pid);
	}

	protected FahrstreifenQuelleASCII(final ConfigurationObject confObject) throws DataModellMismatchException {
		super(confObject);
	}

	@Override
	protected void initData(final ConfigurationObject confObject) {
		final Data oData = confObject.getConfigurationData(FahrstreifenQuelleASCII.__AtgVerkehrsDatenQuelleASC);
		_URL = oData.getTextValue("Url").getText();
		_DatenFormat = oData.getTextValue("DatenFormat").getText();
		_VKRaster = oData.getTextValue("VKRaster").getText();
		_FahrStreifenIndex = oData.getUnscaledValue("FahrStreifenIndex").intValue();
	}

	/**
	 * gibt ein Array aller Objekte zurück, die vom Typ
	 * typ.verkehrsDatenQuelleASC sind.
	 *
	 * @return FahrstreifenQuelleASCII[]
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	public static FahrstreifenQuelleASCII[] getAll() throws DataModellMismatchException {
		final List<SystemObject> list = LogosClientDavConnection.getInstance().getDataModel().getType("typ.verkehrsDatenQuelleASCII")
				.getElements();
		final FahrstreifenQuelleASCII[] retValue = new FahrstreifenQuelleASCII[list.size()];
		for (int i = 0; i < list.size(); i++) {
			retValue[i] = new FahrstreifenQuelleASCII((ConfigurationObject) list.get(i));
		}
		return retValue;
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("PID: \"").append(_PID);
		sb.append("\"  URL: \"").append(_URL);
		sb.append("\"  DatenFormat: \"").append(_DatenFormat);
		sb.append("\"  VKRaster: \"").append(_VKRaster);
		sb.append("\"  FahrStreifenIndex: \"").append(_FahrStreifenIndex);
		sb.append("\"");
		return sb.toString();
	}

	@Override
	public String getTreeString() {
		return getName();
	}

	@Override
	ConfigurationObjectType getType() {
		if (FahrstreifenQuelleASCII.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			FahrstreifenQuelleASCII.__ConfigurationObjectType = (ConfigurationObjectType) model
					.getType("typ.verkehrsDatenQuelleASCII");
			FahrstreifenQuelleASCII.__AtgVerkehrsDatenQuelleASC = model
					.getAttributeGroup("atg.verkehrsDatenQuelleASCII");
			final List<SystemObject> list = FahrstreifenQuelleASCII.__ConfigurationObjectType.getElements();
			model.getConfigurationData(list, FahrstreifenQuelleASCII.__AtgVerkehrsDatenQuelleASC);
		}
		return FahrstreifenQuelleASCII.__ConfigurationObjectType;
	}

}
